package RPM::Source::Convert::Plugins::Mageia2ALT;
use strict;
use warnings;
use Carp;

use RPM::Source::SpecConst;
use RPM::Source::Tools::Dependency::Filter;
use RPM::Source::Transformation::Factory;
use RPM::Source::Transformation::OptHandlerWrapper;
use Source::Shared::Resource::DistroMap;
use RPM::Source::Convert::Plugins::Generic2ALT;
# after Generic2ALT; not to disturb the order
use RPM::Source::Transformation::Factory::DependencyFilter;
use RPM::Source::Convert::Plugins::PerlAutodepsUseStyle2PathStyle;
use RPM::Source::Convert::Plugins::Python;

# rpm5.org srpms in openmandriva and rosa
use RPM::Source::Transformation::RPMEditorProvider;
$RPM::Source::Transformation::RPMEditorProvider::opt_use_rpm2cpio=1;

use Source::Package::Comparators::Raw;
use RPM::Source::Transformation::Factory::RaiseRelease;
$RPM::Source::Transformation::Factory::RaiseRelease::version_comparator=\&Source::Package::Comparators::Raw::_rpm_plus_perl_version_cmp;

our @ISA=qw/RPM::Source::Transformation::Factory/;
push @RPM::Source::TransformContainer::FACTORY_CLASSES, __PACKAGE__;

$DistroMap::default_originrepo='mageia';

# Mageia Linux version (%mgaversion == %distro_release)
# - "9.1"    =>  910
# - "10.2.2" => 1022
my $mgaversion=600;

# %mdkversion (and %mdkver/%mdvver)
my $mdkversion=201100;

# TODO: https://wiki.mageia.org/en/Python_policy

# see rpm-mageia-setup-build for mga-specific macros

&RPM::Source::Transformation::Factory::DependencyFilter::register_filter(
    -name => 'Mageia Perl versions',
    -description => 'convert Mageia specific perl versions',
    -class => 'Mageia2ALT::DependencyFilter::PerlVer',
    -tags => SPEC_TAG_SET_REQUIRES,
    -priority => FILTER_PRIORITY_PREDISTROMAP,
);
&RPM::Source::Transformation::Factory::DependencyFilter::register_filter(
    -name => 'Mageia lib64 names hacks',
    -description => 'hacks for Requires: %{_lib}sdl2_mixer2.0_0',
    -class => 'Mageia2ALT::DependencyFilter::Lib64Names',
    -priority => FILTER_PRIORITY_PREDISTROMAP,
);
&RPM::Source::Transformation::Factory::DependencyFilter::register_filter(
    -name => 'Mageia static-devel names hacks',
    -description => 'hacks for static-devel -> devel-static',
    -class => 'Mageia2ALT::DependencyFilter::StaticDevel',
    -priority => FILTER_PRIORITY_POSTDISTROMAP,
);


sub transformations {
    return 
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_PRECONVERT,
	    -name => 'mageia2alt_preconvert',
	    -opt_handler_sub=>\&opt_handler_mageia2alt_preconvert),
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_CONVERT,
	    -name => 'mageia2alt_convert',
	    -opt_handler_sub=>\&opt_handler_mageia2alt_convert);
}

sub opt_handler_mageia2alt_convert {
    my (undef,$spec,$parent,$opt,$resourceobj)=@_;

    my $macros=$spec->macros();
    my $mainsec=$spec->main_section;

    my $has_perl_convert_version=0;
    my ($flag_need_mageia_compat, $flag_need_rpm_helper, $flag_need_cmake);
    my ($flag_need_mgaversion,$flag_need_mdkversion,$flag_need_distro_release,$flag_need_distro_section,
	$flag_need_distro_branch,$flag_need_product_distribution);

    my @__push_mainsec;
    foreach my $sec ($spec->get_sections()) {
	my $t = $sec->get_type();
	next if $t eq 'changelog';
	if ($t eq 'package') {
	    $sec->map_body(sub {
		# todo use __calculate_mkrel from Generic2ALT
		s/\%mkrel\s+//g;
		s/\%\{mkrel\s+([^}]+)\}/$1/g;
		# %EVRD - see rpm-mageia-setup-build (20build.macros).
		# we do not use distepoch, so replace to %EVR
		s,(?<!%)\%\{?EVRD\b\}?,%{EVR},g;
		# our perl policy is different
		$has_perl_convert_version=1 if s/\%perl_convert_version\s+//g;
		$flag_need_mageia_compat=1 if m/(?<!%)\%\{?rename\s/;
		$flag_need_mageia_compat=1 if m/(?<!%)\%\{?arch_tagged\s/;

#		if (/^BuildRequires:/i) {
#		} elsif (/^Requires:/i) {
#		}
			   } );
	} elsif ($t eq 'prep') {
	    $sec->map_body(sub {
		if (s/(?<!\%)\%apply_patches/#%%apply_patches/) {
		    my $patchlist=$spec->raw_patchlist_ref;
		    for (my $i=0; $i <@$patchlist;$i++) {
			next unless $patchlist->[$i];
			$_.='%patch'.$i.' -p1'."\n";
		    }
		}
		if (s/(?<!%)\%autosetup\b/%setup/) {
		    my $patcharg='';
		    $patcharg=$1 if s/\s(-p\d+)\b/ /;
		    my $patchlist=$spec->raw_patchlist_ref;
		    for (my $i=0; $i <@$patchlist;$i++) {
			next unless $patchlist->[$i];
			$_.='%patch'.$i.' '.$patcharg."\n";
		    }
		}


			   });
	} elsif ($t eq 'build') {
	} elsif ($t eq 'install') {
	    $sec->map_body(sub {
		s,\%make_install(?=(?:\s|$)),%make_install DESTDIR=%buildroot install,g;
		# multiarch-utils
		s,\%multiarch_(binaries|includes)\b,echo #multiarch_$1,g;
			   });
	} elsif ($t eq 'triggerun') {
	    if ($sec->match_body(qr'(?:chkconfig|systemctl)')) {
		$sec->delete;
		next;
	    }
	} elsif ($RPM::Source::SpecConst::SCRIPT_SECTION{$t}) {
	    $sec->map_body(sub {
		$flag_need_mageia_compat=1 if m,(?<!%)\%\{?(?:_tmpfilescreate|__id_u)\b,;
		$flag_need_rpm_helper=1 if m,(?<!%)\%(?:
_post_service|_preun_service|_postun_service|_post_unit|_preun_unit|
_get_password|create_ghostfile|_create_ssl_certificate|
_pre_useradd|_postun_userdel|_pre_groupadd|_postun_groupdel|
_post_shelladd|_preun_shelldel|_post_syslogadd|_preun_syslogdel|
_post_webapp|_postun_webapp
),x;
			   } );
	} elsif ($t eq 'files') {
	    #
#%{_libdir}/*.so.%{major}{,.*}
	    $sec->map_body(sub {
		if (s!(\%\{_libdir\}/\*\.so\.\%\{major\}\{,\.\*\})!%{_libdir}/*.so.*!g) {
		    warn "INFO: $1 not supported in rpm 4.0.4 -- reduced.\n"; # rpm404
		}
		# hack; found in perl-B-C(-devel)
		s,\%\{perl_root\}/\%\{perl_version\}/\%\{full_arch\},%{perl_vendor_archlib},;

		# mga icons correction
		s,\%\{?_iconsdir\}?/([^/]+\.(?:png|xcf|svg|xpm)),%_niconsdir/$1,;

		# multiarch, see SDLmm, 
		s,%multiarch\b,#multiarch,;

		# ruby support
		s,\%\{ruby_vendorarchdir\},%{ruby_sitearchdir},g;
		#%ruby_sitelibdir

		s,(?<!%)\%\{?python3_version\b\}?,%{__python3_version},g;
		    
		$flag_need_mageia_compat=1 if m,(?<!%)\%\{?(?:_extension)\b,;
			   } );

	}

	# prep build install
	$sec->map_body(sub {
	    # clean up
	    s,\%\{\??perl_default_filter\},,g;
	    s,\%setup_compile_flags\b,,g;
	    
	    #cp -af %{_rpmconfigdir}/config.{guess,sub}
	    s,(?:\%\{?_rpmconfigdir\}?|\%\{_usr\}/lib/rpm|/usr/lib/rpm)/config\.,/usr/share/gnu-config/config.,g;
	    
	    s,\%configure2_5x,%configure,g;
	    s,\%compat_makeinstall_std,%makeinstall_std,g;
	    s,\%\{compat_makeinstall_std\},%{makeinstall_std},g;
	    if (s/(?<!%)\%\{?cmake\}?(?=\s|$)/%{mageia_cmake}/g) {
		$flag_need_mageia_compat=1;
		$flag_need_cmake=1;
	    }
	    if (s/(?<!%)\%\{?cmake_qt([45])\}?(?=\s|$)/%{mageia_cmake_qt$1}/g) {
		$flag_need_mageia_compat=1;
		$flag_need_cmake=1;
	    }
	    $flag_need_cmake=1 if s,\%\{__cmake\},cmake,g;
	    # not defined in alt
	    s/(?<!%)\%\{?ldflags(?:\}|\b)//g;

	    $flag_need_mageia_compat=1 if /(?<!%)
		\%(?:qmakeinstall_std
		| configure_scons
		| configure_scons_kde3
		| scons
		| scons_install
		| serverbuild
		)\b
	      /x;

	    # or use rpm-macros-fonts
	    s,\%\{_xfontdir\}/TTF,%\{_datadir\}/fonts/ttf,g;
	    s,/usr/sbin/strfile,/usr/bin/strfile,g;

	    # mga icons correction
	    s,\%\{?_iconsdir\}?/(\%\{?name\}?\.png),%_niconsdir/$1,;
	    if (/mkdir/) {
		# mkdir -p %buildroot{%_liconsdir,%_miconsdir}
		s/\{\%\{?_liconsdir\}?,\%\{?_miconsdir\}?\}/{%_liconsdir,%_miconsdir,%_niconsdir}/;
		s/\{\%\{?_miconsdir\}?,\%\{?_liconsdir\}?\}/{%_liconsdir,%_miconsdir,%_niconsdir}/;
	    }


	} ) if ($t eq 'install' or $t eq 'build' or $t eq 'prep');

	# all sections
	$sec->map_body(sub {
	    # used in desktop files as vendor
	    s/(?<!%)\%_real_vendor\b/%_vendor/g;
	    s/(?<!%)\%\{_real_vendor\}/%{_vendor}/g;
	    s/\%\{Summary\}/%{summary}/g;
	    # hack
	    s/\%pyver\b/27/g;
	    s/\%\{pyver\}/27/g;
	    $flag_need_mgaversion=1 if /(?<!%)\%\{?mgaversion\b/;
	    $flag_need_mdkversion=1 if /(?<!%)\%\{?(?:mdkversion|mdkver|mdaver)\b/;
	    $flag_need_mageia_compat=1 if /(?<!%)\%\{?py_(?:plat|pure)(?:lib|site)dir\b/;
	    $flag_need_product_distribution=1 if /(?<!%)\%\{?product_distribution\b/;
	    # %if "%{distro_section}" == "tainted"
	    $flag_need_distro_section=1 if m/(?<!%)\%\{?distro_section\b/;
	    # %if "%distro_branch" == "Cauldron"
	    $flag_need_distro_branch=1 if m/(?<!%)\%\{?distro_branch\b/;
	    $flag_need_distro_release=1 if /(?<!%)\%\{?distro_release\b/;
			   } );


    }
    $mainsec->push_body(@__push_mainsec) if @__push_mainsec;
    if ($has_perl_convert_version) {
	my $macros=$spec->macros();
	my $ver=$macros->get_raw('version');
	$ver=~s/^(?:\%perl_convert_version)?\s+//;
	$macros->set_raw('version',$ver);
    }
    $mainsec->unshift_body('%define product_distribution Mageia'."\n") if $flag_need_product_distribution;
    $mainsec->unshift_body('%define distro_section tainted'."\n") if $flag_need_distro_section;
    $mainsec->unshift_body('%define distro_branch Cauldron'."\n") if $flag_need_distro_branch;
    $mainsec->unshift_body('%define distro_release '.$mgaversion."\n") if $flag_need_distro_release;
    $mainsec->unshift_body('%define mgaversion '.$mgaversion."\n") if $flag_need_mgaversion;
    $mainsec->unshift_body(
	'%define mdkversion '.$mdkversion."\n",
	'%define mdkver '.$mdkversion."\n",
	'%define mdvver '.$mdkversion."\n",
	) if $flag_need_mdkversion;
	    
    my $buildreq=$resourceobj->buildreq;
    $buildreq->add_pre_buildreq('rpm-helper') if $flag_need_rpm_helper;
    $buildreq->add_pre_buildreq('rpm-macros-mageia-compat') if $flag_need_mageia_compat;
    $buildreq->add_buildreq('cmake') if $flag_need_cmake; # like kde apps ...

    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

sub opt_handler_mageia2alt_preconvert {
    my (undef,$spec,$parent,$opt,$resourceobj)=@_;
    # mklibname should be handled before DistroMapping deps
    foreach my $sec ($spec->get_sections()) {
	my $t = $sec->get_type();
	my $headlineref=$sec->get_headline_ref();
	&__subst_mklibname($headlineref) if $headlineref; # skips main package
	# files - found in texlive, description -- libqxt
	if ($t eq 'package' or $t eq 'files' or $t eq 'description') {
	    $sec->map_body(sub {
		&__subst_mklibname(\$_);
	    });
	}
    }
    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

# https://wiki.mageia.org/en/Libraries_policy
#%mklibname [-d [-s]] name [[api] major]
#-d - generate a name for a devel package
#-s - generate a name for a static package (to be used together with -d)
#name - the library name (note that if the library name is libfoo, you should enter "foo", not "libfoo")
#major - the major number to be appended into the name (this should not be used with -d, except in packages mentioned in special cases below)
#api - if the library has a SONAME like libfoo-1.2.so.4, api should be set to 1.2 and major to 4. This results in libfoo1.2_4

sub __subst_mklibname {
    my ($ref)=shift;
    # mandrake mklibname macros
    #print STDERR "__subst_mklibname: ref=$$ref";

    $$ref=~s/\%mklibname((?:\s+\S+)+)/&__mklibname1($1)/ge;
    $$ref=~s/\%\{mklibname((?:\s+(?:-d|-s|[\d\._\w]+|[\d\._\w]*\%\{[\d\._\w]+\}|[\d\._\w]*\%[\d\._\w]+))+)\}/&__mklibname1($1)/ge;
    #print STDERR "__subst_mklibname: got=$$ref";
}

sub __mklibname1 {
    my $in=shift;
    #print STDERR "__mklibname1: in=$in\n";
    my @args=grep {defined ($_) and $_!~/^\s*$/} split(/\s+/,$in);
    return &__mklibname(@args);
}

# add bad cases to t/mklibname.t !!!
sub __mklibname {
    my (@args)=@_;
    #@args=map {&__brace_macro($_)} grep {defined $_} @args;
    my ($is_devel,$is_static);
    my @body;
    foreach my $arg (@args) {
	if ($arg eq '-d') {
	    $is_devel=1;
	} elsif($arg eq '-s') {
	    $is_static=1;
	} else {
	    push @body, &__wrap_macro($arg);
	}
    }
    my $out;
    if (@body>3) {
	confess "__mklibname: too much args: ",join(',',@args);
    } elsif (@body==3) {
	my $api=$body[1];
	my $major=$body[2];
	$out='lib'.$body[0].$api.'_'.$major;
    } else {
	$out='lib'.join('',@body);
    }
    if ($is_devel) {
	$out=$out.'-devel';
    }
    if ($is_static) {
	$out=$out.'-static';
    }
    return $out;
}

sub __wrap_macro {
    my $in=shift;
    if (defined ($in) and substr($in,0,1) eq '%') {
	$in ='%{'.substr($in,1).'}' if substr($in,1,1) ne '{';
    }
    return $in;
}

# drop perl deps in mga format: like
# E: Версия >='2.190.0' для 'perl-autodie' не найдена
package Mageia2ALT::DependencyFilter::PerlVer;
use strict;
use warnings;
our @ISA=q/RPM::Source::Tools::Dependency::Filter/;

sub apply {
    my ($self, $tagobj, $dep) = @_;
    my $pkgname=$dep->{-name};
    return $dep if $pkgname!~/^perl[\(-]/;
    $dep->{-version}=undef;
    return $dep;
}

# fix Requires:     %{_lib}sdl2_mixer2.0_0
package Mageia2ALT::DependencyFilter::Lib64Names;
use strict;
use warnings;
our @ISA=q/RPM::Source::Tools::Dependency::Filter/;

sub apply {
    my ($self, $tagobj, $dep) = @_;
    $dep->{-name}=~s/^\%\{_lib\}/lib/;
    return $dep;
}

package Mageia2ALT::DependencyFilter::StaticDevel;
use strict;
use warnings;
our @ISA=q/RPM::Source::Tools::Dependency::Filter/;

sub apply {
    my ($self, $tagobj, $dep) = @_;
    $dep->{-name}=~s/-static-devel$/-devel-static/;
    return $dep;
}

1;
__END__
